/**
 * components.js
 * Responsável por verificações de conexão.
 */

var Components = {}

// Configurações
Components.config = {
	// Utilizado pelas verificações
	isOnline: 1,
	// Outras configs
	mode: 'development', // development para receber erros com ALERT
	alerts: { "error": true, "warning": true, "success": true, "info": true }
}

// Observa mudanças na rede
window.addEventListener('online', function(e) {
	Components.setInternetStatus(1);
});

window.addEventListener('offline', function(e) {    
	Components.setInternetStatus(0);
});
	
/**
 * init
 * Inicia a contagem/timer de a cada 10 seg verificar algo
 */
Components.init = function() {

		/*
		 * Verificação de status da internet (Se está online)
		 */
		if(!Components.config.isOnline) {
			Components.showInAppNotification('error', 'Detectamos problemas com sua conexão.');
			return;
		} else {
			Components.removeInAppNotification('error');
		}

		 /*
         * Verificação do GPS
         */

        CheckGPS.check(function(){
		    Components.removeInAppNotification('error');
		 },
		function(){
		    Components.showInAppNotification('error', 'Sinal GPS não encontrado.');
		});

		/*
		 * Pega o tipo de conexão, caso não seja 3G, 4G ou Wi-Fi, fala que está lenta/problemas
		 */
		var connectionType = Components.getNetworkType();	
		if( connectionType == Connection.ETHERNET || connectionType == Connection.CELL_2G || connectionType == Connection.CELL || connectionType == Connection.NONE ) {
			Components.showInAppNotification('error', 'Detectamos problemas com sua conexão.');
		} else {
			Components.removeInAppNotification('error');
		}

}


/**
 * getInternetStatus
 * Retorna o status da internet (Se o usuario está conectado)
 */
Components.setInternetStatus = function(status) {
	Components.config.isOnline = status;
	return Components.config.isOnline;
}

/**
 * getNetworkType
 * Retorna o tipo de conexão (2G, 3G, etc)
 */ 
Components.getNetworkType = function() {
	return navigator.connection.type;
}

/**
 * Adiciona ou remove classes a notificação (troca a cor)
 */
Components.setNotificationStatus = function(method, className, message) {
	var notificationElement = document.getElementById('alert-message');
	notificationElement.innerHTML = message;

	if(method == 'add')
		notificationElement.classList.add(className);
	else if(method == 'remove')
		notificationElement.classList.remove(className);
}

/**
 * showAlert
 * Verifica a plataforma e faz um alerta de acordo
 */
Components.showAlert = function(message, opt) {
	if(this.config.mode == 'development') {
		alert(message);
	}
}

/**
 * Verifica se o parâmetro (tipo de status) é válido
 */
Components.statusIsValid = function(type) {
	if(typeof this.config.alerts[type] == 'undefined') {
		this.showAlert('Parâmetro "'+ type +'" inválido.')
		return false;
	}

	return true;
}
/**
 * Mostra notificações nativamente
 */
Components.showInAppNotification = function(type, message) {

	if(!this.statusIsValid(type))
		return;

	Components.setNotificationStatus('add', type, message);

}

Components.removeInAppNotification = function(type) {

	if(!this.statusIsValid(type))
		return;

	Components.setNotificationStatus('remove', type, '');
}

Components.init();

